@@ -1,6 +1,6 @@
 package Ubic::Daemon;
-{
-  $Ubic::Daemon::VERSION = '1.37_02';
+BEGIN {
+  $Ubic::Daemon::VERSION = '1.37_03';
 }
 
 use strict;
@@ -10,7 +10,7 @@ use warnings;
 
 
 use IO::Handle;
-use POSIX qw(setsid);
+use POSIX qw(setsid :sys_wait_h);
 use Time::HiRes qw(sleep);
 use Params::Validate qw(:all);
 use Carp;
@@ -61,6 +61,32 @@ sub _log {
     print {$fh} '[', scalar(localtime), "]\t$$\t", @_, "\n";
 }
 
+sub _log_exit_code {
+    my ($fh, $code, $pid) = @_;
+    if ($code == 0) {
+        _log($fh, "daemon $pid exited");
+        return;
+    }
+
+    my $msg = "daemon $pid failed with \$? = $?";
+    if (my $signal = $? & 127) {
+        my $signame = _signame($signal);
+        if (defined $signame) {
+            $msg = "daemon $pid failed with signal $signame ($signal)";
+        }
+        else {
+            $msg = "daemon $pid failed with signal $signal";
+        }
+    }
+    elsif ($? & 128) {
+        $msg = "daemon $pid failed, core dumped";
+    }
+    elsif (my $code = $? >> 8) {
+        $msg = "daemon $pid failed, exit code $code";
+    }
+    _log($fh, $msg);
+}
+
 sub stop_daemon($;@) {
     my ($pidfile, @tail) = validate_pos(@_, { type => SCALAR }, 0);
     my $options = validate(@tail, {
@@ -153,12 +179,11 @@ sub start_daemon($) {
         my $ubic_fh;
         my $lock;
         my $instant_exit = sub {
-            my $status = shift; # nobody cares for this status anyway...
             close($ubic_fh) if $ubic_fh;
             STDOUT->flush;
             STDERR->flush;
             undef $lock;
-            POSIX::_exit($status); # don't allow any cleanup to happen - this process was forked from unknown environment, don't want to run unknown destructors
+            POSIX::_exit(0); # don't allow any cleanup to happen - this process was forked from unknown environment, don't want to run unknown destructors
         };
 
         eval {
@@ -215,14 +240,25 @@ sub start_daemon($) {
             if ($child = fork) {
                 # guardian
 
-                my $child_guid = $OS->pid2guid($child);
-                die "Can't detect guid" unless $child_guid;
-                _log($ubic_fh, "child guid: $child_guid");
-                $pid_state->write({ pid => $child, guid => $child_guid });
-
                 _log($ubic_fh, "guardian pid: $$");
                 _log($ubic_fh, "daemon pid: $child");
 
+                my $child_guid = $OS->pid2guid($child);
+                unless ($child_guid) {
+                    if ($OS->pid_exists($child)) {
+                        die "Can't detect guid";
+                    }
+                    $? = 0;
+                    unless (waitpid($child, WNOHANG) == $child) {
+                        die "No pid $child but waitpid didn't collect $child status";
+                    }
+                    _log_exit_code($ubic_fh, $?, $child);
+                    $pid_state->remove();
+                    die "daemon exited immediately";
+                }
+                _log($ubic_fh, "child guid: $child_guid");
+                $pid_state->write({ pid => $child, guid => $child_guid });
+
                 my $kill_sub = sub {
                     if ($term_timeout) {
                         _log($ubic_fh, "SIGTERM timeouted after $term_timeout second(s)");
@@ -231,7 +267,7 @@ sub start_daemon($) {
                     kill -9 => $child;
                     _log($ubic_fh, "daemon $child probably killed by SIGKILL");
                     $pid_state->remove();
-                    $instant_exit->(0);
+                    $instant_exit->();
                 };
 
                 my $sigterm_sent;
@@ -249,39 +285,19 @@ sub start_daemon($) {
                 };
                 print {$write_pipe} "pidfile written\n" or die "Can't write to pipe: $!";
                 close $write_pipe or die "Can't close pipe: $!";
+                undef $write_pipe;
 
                 $? = 0;
                 waitpid($child, 0);
-                if ($? > 0) {
-                    my $msg = "daemon $child failed with \$? = $?";
-                    if (my $signal = $? & 127) {
-                        if ($sigterm_sent && $signal == &POSIX::SIGTERM) {
-                            # it's ok, we probably sent this signal ourselves
-                            _log($ubic_fh, "daemon $child exited by sigterm");
-                            $pid_state->remove;
-                            $instant_exit->(0);
-                        }
-                        my $signame = _signame($signal);
-                        if (defined $signame) {
-                            $msg = "daemon $child failed with signal $signame ($signal)";
-                        }
-                        else {
-                            $msg = "daemon $child failed with signal $signal";
-                        }
-                    }
-                    elsif ($? & 128) {
-                        $msg = "daemon $child failed, core dumped";
-                    }
-                    elsif (my $code = $? >> 8) {
-                        $msg = "daemon $child failed, exit code $code";
-                    }
-                    _log($ubic_fh, $msg);
-                    $pid_state->remove;
-                    $instant_exit->(1);
+                my $code = $?;
+                if ($sigterm_sent and ($code & 127) == &POSIX::SIGTERM) {
+                    # it's ok, we probably sent this signal ourselves
+                    _log($ubic_fh, "daemon $child exited by sigterm");
+                }
+                else {
+                    _log_exit_code($ubic_fh, $code, $child);
                 }
-                _log($ubic_fh, "daemon $child exited");
                 $pid_state->remove;
-                $instant_exit->(0);
             }
             else {
                 # daemon
@@ -304,6 +320,7 @@ sub start_daemon($) {
 
                 print {$write_pipe} "execing into daemon\n" or die "Can't write to pipe: $!";
                 close($write_pipe) or die "Can't close pipe: $!";
+                undef $write_pipe;
 
                 # finally, run underlying binary
                 if (ref $bin) {
@@ -321,7 +338,7 @@ sub start_daemon($) {
             print {$write_pipe} "Error: $@\n";
             $write_pipe->flush;
         }
-        $instant_exit->(1);
+        $instant_exit->();
     }
     waitpid($child, 0); # child should've exited immediately
     close($write_pipe) or die "Can't close write_pipe: $!";
@@ -411,7 +428,7 @@ Ubic::Daemon - daemon management utilities
 
 =head1 VERSION
 
-version 1.37_02
+version 1.37_03
 
 =head1 SYNOPSIS
 
