@@ -1,8 +1,11 @@
 package Ubic::Lockf::Alarm;
-{
-  $Ubic::Lockf::Alarm::VERSION = '1.37_02';
+BEGIN {
+  $Ubic::Lockf::Alarm::VERSION = '1.37_03';
 }
 
+use strict;
+use warnings;
+
 # ABSTRACT: alarm guard
 
 sub new ($$) {
@@ -34,7 +37,7 @@ Ubic::Lockf::Alarm - alarm guard
 
 =head1 VERSION
 
-version 1.37_02
+version 1.37_03
 
 =head1 DESCRIPTION
 
